import 'package:intl/intl.dart';
import 'package:intl/date_symbol_data_local.dart';
import 'string_extension.dart';

extension DateExtension on DateTime {
  String formatString({String format = 'MM.dd yyyy', String locale = 'en'}) {
    initializeDateFormatting(locale); // ja_JP
    final formatter = DateFormat(format, locale);
    return formatter.format(this);
  }

  // return this - date, also check this.date is wheter in UTC format or not
  int diffDays(DateTime date, {bool bUTC = false}) => bUTC ? toUtc().difference(date.toUtc()).inDays : this.difference(date).inDays;
  int diffHours(DateTime date, {bool bUTC = false}) => bUTC ? toUtc().difference(date.toUtc()).inHours : this.difference(date).inHours;
  int diffMinutes(DateTime date, {bool bUTC = false}) => bUTC ? toUtc().difference(date.toUtc()).inMinutes : this.difference(date).inMinutes;
  int diffSeconds(DateTime date, {bool bUTC = false}) => bUTC ? toUtc().difference(date.toUtc()).inSeconds : this.difference(date).inSeconds;

  // this >= date => true
  // this < date => false
  bool isCompare(DateTime date, {bool bUTC = false}) => bUTC ? toUtc().compareTo(date.toUtc()) != -1 || false : this.compareTo(date) != -1 || false;

  String beautifyTime(
      {bool bUTC = false,
      String strYear = '{0} year',
      String strMonth = '{0} month',
      String strDay = '{0} day',
      String strHour = '{0} hour',
      String strMinute = '{0} minute',
      String strJustNow = 'Just now',
      String strSuffix = 'ago',
      bool bAddPlural = true,
      bool bShowTwoFields = false}) {
    var curDate = DateTime.now();
    if (bUTC) curDate = DateTime.now().toUtc();

    DateTime lastUpdatedDate = this;
    var diff = curDate.difference(lastUpdatedDate);

    // Supports expiry date format
    if (diff.inSeconds < 0) diff = lastUpdatedDate.difference(curDate);

    int years = (diff.inDays / 365).floor();
    int months = (diff.inDays / 30).floor();
    int days = diff.inDays.remainder(30);
    int hours = diff.inHours.remainder(24);
    int minutes = diff.inMinutes.remainder(60);
    strSuffix = ' ' + strSuffix;

    if (bAddPlural) {
      if (years > 1) strYear += 's';
      if (months > 1) strMonth += 's';
      if (days > 1) strDay += 's';
      if (hours > 1) strHour += 's';
      if (minutes > 1) strMinute += 's';
    }

    if (years > 0) {
      months -= 12;
      if (bShowTwoFields && months > 0) {
        String strNewDate = strYear + ' ' + strMonth.replaceAll('{0}', '{1}');
        return strNewDate.format([years.toString(), months.toString()]) + strSuffix;
      } else
        return strYear.format([years.toString()]) + strSuffix;
    } else if (months > 0) {
      if (bShowTwoFields && days > 0) {
        String strNewDate = strMonth + ' ' + strDay.replaceAll('{0}', '{1}');
        return strNewDate.format([months.toString(), days.toString()]) + strSuffix;
      } else
        return strMonth.format([months.toString()]) + strSuffix;
    } else if (days > 0) {
      if (bShowTwoFields && hours > 0) {
        String strNewDate = strDay + ' ' + strHour.replaceAll('{0}', '{1}');
        return strNewDate.format([days.toString(), hours.toString()]) + strSuffix;
      } else
        return strDay.format([days.toString()]) + strSuffix;
    } else if (hours > 0) {
      if (bShowTwoFields && minutes > 0) {
        String strNewDate = strHour + ' ' + strMinute.replaceAll('{0}', '{1}');
        return strNewDate.format([hours.toString(), minute.toString()]) + strSuffix;
      } else
        return strHour.format([hours.toString()]) + strSuffix;
    } else if (minutes > 0) {
      return strMinute.format([minutes.toString()]) + strSuffix;
    } else
      return strJustNow;
  }
}
