import 'package:flutter/material.dart';

/// WidgetX extensions to power our flutter widgets
/// WidgetX extension is now integrated in Get package and don't need from this package anymore
/// Highly experimental. API not stable yet.
extension WidgetX on Widget {
  Widget container({Key key = const Key('no_key'), required EdgeInsets margin, required double width, required double height}) {
    return Container(
      key: key,
      child: this,
      margin: margin,
    );
  }

  Widget sizedBox({Key key = const Key('no_key'), required double width, required double height}) {
    return SizedBox(
      key: key,
      child: this,
      width: width,
      height: height,
    );
  }

  Widget scaffold({
    Key key = const Key('no_key'),
    required PreferredSizeWidget appbar,
    required Widget floatingActionButton,
  }) {
    return Scaffold(
      key: key,
      body: this,
      appBar: appbar,
      floatingActionButton: floatingActionButton,
    );
  }

  Widget center({Key key = const Key('no_key')}) {
    return Center(
      child: this,
      key: key,
    );
  }

  Widget padding(
    EdgeInsets padding, {
    Key key = const Key('no_key'),
  }) {
    return Padding(
      key: key,
      padding: padding,
      child: this,
    );
  }

//  Widget paddingAll(double padding) => Padding(padding: EdgeInsets.all(padding), child: this);
//
//  Widget paddingOnly({double left = 0, double top = 0, double right = 0, double bottom = 0}) =>
//      Padding(padding: EdgeInsets.only(left: left, top: top, right: right, bottom: bottom), child: this);
//
//  Widget paddingSymmetric({double horizontal = 0, double vertical = 0}) =>
//      Padding(padding: EdgeInsets.symmetric(horizontal: horizontal, vertical: vertical), child: this);
}

extension OnPressed on Widget {
  Widget ripple(Function? onPressed, {BorderRadiusGeometry borderRadius = const BorderRadius.all(Radius.circular(5))}) => Stack(
        children: <Widget>[
          this,
          Positioned(
              left: 0,
              right: 0,
              top: 0,
              bottom: 0,
              child: TextButton(
                onPressed: () {
                  if (onPressed != null) {
                    onPressed();
                  }
                },
                child: Container(),
              ))
        ],
      );
}

/// Extensions on string to create Text widget
/// TODO: add rich text support extensions
extension TextX on String {
  Widget text({
    Key key = const Key('no_key'),
    required TextStyle style,
  }) {
    return Text(
      this,
      key: key,
      style: style,
    );
  }

  Widget textCenter({
    Key key = const Key('no_key'),
  }) {
    return Text(
      this,
      key: key,
      textAlign: TextAlign.center,
    );
  }

  Widget textLeft({
    Key key = const Key('no_key'),
  }) {
    return Text(
      this,
      key: key,
      textAlign: TextAlign.left,
    );
  }

  Widget textRight({
    Key key = const Key('no_key'),
  }) {
    return Text(
      this,
      key: key,
      textAlign: TextAlign.right,
    );
  }

  Widget textJustify({
    Key key = const Key('no_key'),
  }) {
    return Text(
      this,
      key: key,
      textAlign: TextAlign.justify,
    );
  }

  Widget networkImage({
    Key key = const Key('no_key'),
    required BoxFit fit,
  }) {
    return Image.network(
      this,
      fit: fit,
    );
  }

  //// Create circle image avatar from network image
  Widget circleAvatar({
    Key key = const Key('no_key'),
    required double radius,
    required Color backgroundColor,
  }) {
    return CircleAvatar(
      radius: radius,
      backgroundImage: NetworkImage(this),
      backgroundColor: backgroundColor,
    );
  }
}

/// Extensions on List<String> to convert to List<Widget> with texts
///
/// Usage:
///
/// ```dart
/// List<String> fruits = ['apple', 'orange', 'grapes'];
///
/// Container(
///   child: fruits.texts()
/// )
/// ```
extension ListTextX on List<String> {
  List<Widget> texts({
    required TextStyle style,
  }) {
    return List<Widget>.generate(
      this.length,
      (index) => this[index].text(style: style, key: Key('')),
    );
  }
}

/// Extensions on list of Widgets
/// Methods: `row()`, `column()`, `stack()`
extension WidgetListX on List<Widget> {
  Row row({
    Key key = const Key('no_key'),
    MainAxisAlignment mainAxisAlignment = MainAxisAlignment.start,
    CrossAxisAlignment crossAxisAlignment = CrossAxisAlignment.center,
  }) {
    return Row(
      key: key,
      children: this,
      mainAxisAlignment: mainAxisAlignment,
      crossAxisAlignment: crossAxisAlignment,
    );
  }

  Column column({
    Key key = const Key('no_key'),
    MainAxisAlignment mainAxisAlignment = MainAxisAlignment.start,
    CrossAxisAlignment crossAxisAlignment = CrossAxisAlignment.center,
  }) {
    return Column(
      key: key,
      children: this,
      mainAxisAlignment: mainAxisAlignment,
      crossAxisAlignment: crossAxisAlignment,
    );
  }

  Stack stack({
    Key key = const Key('no_key'),
  }) {
    return Stack(
      key: key,
      children: this,
    );
  }
}

/// Extensions on alignment of widgets
extension AlignX on Widget {
  Widget centerLeft() {
    return Align(
      child: this,
      alignment: Alignment.centerLeft,
    );
  }

  Widget centerRight() {
    return Align(
      child: this,
      alignment: Alignment.centerRight,
    );
  }

  Widget topCenter() {
    return Align(
      child: this,
      alignment: Alignment.topCenter,
    );
  }

  Widget topLeft() {
    return Align(
      child: this,
      alignment: Alignment.topLeft,
    );
  }

  Widget topRight() {
    return Align(
      child: this,
      alignment: Alignment.topRight,
    );
  }

  Widget bottomCenter() {
    return Align(
      child: this,
      alignment: Alignment.bottomCenter,
    );
  }

  Widget bottomLeft() {
    return Align(
      child: this,
      alignment: Alignment.bottomLeft,
    );
  }

  Widget bottomRight() {
    return Align(
      child: this,
      alignment: Alignment.bottomRight,
    );
  }
}
